<?
/* Name: diagram.php
 * Project: Remote Sensor based on AVR USB driver
 * Author: Christian Starkjohann
 * Creation Date: 2005-04-13
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: diagram.php 314 2007-03-29 17:17:09Z cs $
 */

include_once("functions.php");

/*
General Description:
This script generates temperature/humidity diagrams from data in .csv files
on the fly. It accepts the following http queries:

  endTime=<decimal integer>
  interval=<string>
  st0, st1, ....
  srh0, srh1, ...

'endTime' specifies the right-most time in the interval. It is specified as
Unix time stamp (seconds since 1970). 'interval' determines the time scale
used. It can be one of the strings 'day', 'week', 'month' or 'year'. Depending
on the interval, the appropriate averaged data file is used. Averaged files
are generated by the 'summary.php' script which should be called immediately
before diagrams are requested. If 'st0', 'st1', ... is set to a non-zero value,
the temperature curve for the related sensor is suppressed.
Likewise, if 'srh0', 'srh1', ... is set to a non-zero value, the relative
humidity curve of the related sensor is suppressed.
*/

/* ------------------------------------------------------------------------- */

function diagramSetup($width, $height, $border)
{
global  $gImage, $gBorder, $gWidth, $gHeight, $gBlack, $gWhite, $gDataColors;

    $gImage = imagecreate($width, $height);
    $gBorder = $border;
    $gWidth = $width;
    $gHeight = $height;
    $gWhite = imagecolorallocate($gImage, 255, 255, 255);   /* background color */
    $gBlack = imagecolorallocate($gImage, 0, 0, 0);
    imagesetstyle($gImage, array($gBlack, $gWhite, $gWhite));
    for($i=0;$i<16;$i++){
        for($h=0;$h<2;$h++){
            $c = colorForSensor($i, $h);
            $gDataColors[2 * $i + $h] = imagecolorallocate($gImage, $c[0], $c[1], $c[2]);
        }
    }
}

function diagramPhysX($x)
{
global  $gWidth, $gHeight, $gBorder;

    return $gBorder + $x * ($gWidth - 2 * $gBorder);    /* normalize into diagram */
}

function diagramPhysY($x)
{
global  $gWidth, $gHeight, $gBorder;

    return $gHeight - $gBorder - $x * ($gHeight - 2 * $gBorder);    /* normalize into diagram */
}

function diagramNextPoint($x, $y, $dataSet) /* $x, $y is in range 0...1 */
{
global  $gImage, $gWidth, $gHeight, $gLastPoint, $gDataColors;

    $x = diagramPhysX($x);  /* normalize into diagram */
    $y = diagramPhysY($y);
    if(isset($gLastPoint[$dataSet])){
        imageline($gImage, $gLastPoint[$dataSet][0], $gLastPoint[$dataSet][1], $x, $y, $gDataColors[$dataSet]);
    }else{
        imageline($gImage, $x, $y, $x, $y, $gDataColors[$dataSet]); /* draw at least a point */
    }
    $gLastPoint[$dataSet][0] = $x;
    $gLastPoint[$dataSet][1] = $y;
}

function diagramAutoscaleSetup($yAxisIndex, $minVal, $maxVal)
{
global  $gMinY, $gMaxY;

    $gMinY[$yAxisIndex] = $minVal;
    $gMaxY[$yAxisIndex] = $maxVal;
}

function diagramAutoscaleForValue($yAxisIndex, $val)
{
global  $gMinY, $gMaxY;

    if($gMinY[$yAxisIndex] > $val)
        $gMinY[$yAxisIndex] = $val;
    else if($gMaxY[$yAxisIndex] < $val)
        $gMaxY[$yAxisIndex] = $val;
}

function diagramScaleTime($val)
{
global $gTimeRange, $gStartTime;

    return ($val - $gStartTime) / $gTimeRange;
}

function diagramAutoscaleValue($yAxisIndex, $val)
{
global  $gMinY, $gMaxY;

    return ($val - $gMinY[$yAxisIndex]) / ($gMaxY[$yAxisIndex] - $gMinY[$yAxisIndex]);
}

function diagramDrawYAxis($yAxisIndex, $name, $fontsize)
{
global  $gImage, $gWidth, $gHeight, $gBorder, $gBlack, $gMinY, $gMaxY;

    /* normalize range to multiples of 10 */
    $gMinY[$yAxisIndex] = (int)floor(($gMinY[$yAxisIndex] / 10 + 0.001)) * 10;
    $gMaxY[$yAxisIndex] = (int)ceil(($gMaxY[$yAxisIndex] / 10 - 0.001)) * 10;

    imagestring($gImage, $fontsize, 2 + $yAxisIndex * ($gWidth - $gBorder - 6), 2, $name, $gBlack);
    for($i = $gMinY[$yAxisIndex]; $i <= $gMaxY[$yAxisIndex]; $i++){
        $y = diagramPhysY(diagramAutoscaleValue($yAxisIndex, $i));
        if($i % 10 == 0){
            $tw = 3;
            imagestring($gImage, $fontsize, 2 + $yAxisIndex * ($gWidth - $gBorder + 2), $y - 7, sprintf("%.0f", $i), $gBlack);
            imageline($gImage, $gBorder, $y, $gWidth - $gBorder, $y, IMG_COLOR_STYLED);
        }else{
            if($i % 5 == 0){
                $tw = 3;
            }else{
                $tw = 1;
            }
        }
        $x = $yAxisIndex == 0 ? $gBorder : $gWidth - $gBorder;
        imageline($gImage, $x - $tw, $y, $x + $tw, $y, $gBlack);
    }
    imageline($gImage, $gBorder, $gBorder, $gWidth - $gBorder, $gBorder, $gBlack);
    imageline($gImage, $gBorder, $gHeight - $gBorder, $gWidth - $gBorder, $gHeight - $gBorder, $gBlack);
}

function diagramDrawXAxis($endTime, $interval, $fontsize)
{
global  $gImage, $gWidth, $gHeight, $gBorder, $gBlack, $gTimeRange, $gStartTime;

    switch($interval){
    case "day":
        $modulo = 3600;
        $gTimeRange = 86400;
        $mag = 0;
        break;
    case "week":
        $modulo = 43200;
        $gTimeRange = 86400 * 7;
        $mag = $endTime - 86400;
        break;
    case "month":
        $modulo = 86400;
        $gTimeRange = 86400 * 30.5;
        $mag = $endTime - 7 * 86400;
        break;
    case "year":
        $modulo = (int)((86400 * 365.24) / 12 + 0.5);   /* approx 1 month */
        $gTimeRange = 86400 * 365.24;
        $mag = $endTime - 86400 * 30.5;
        break;
    }
    $gmtOffset = date('Z', $endTime);
    $gStartTime = $endTime - $gTimeRange;
    $start = (int)($gStartTime / $modulo + 0.999) * $modulo - ($gmtOffset % $modulo);
    $y = $gHeight - $gBorder;
    for($i=$start;$i<$endTime;$i+=$modulo){
        $x = diagramPhysX(diagramScaleTime($i));
        imageline($gImage, $x, $gHeight - $gBorder, $x, $gBorder, IMG_COLOR_STYLED);
        switch($interval){
        case "day":
            imageline($gImage, $x, $y - 3, $x, $y + 3, $gBlack);
            imagestring($gImage, $fontsize, $x - 4, $y + 2, date('H', $i), $gBlack);
            break;
        case "week":
            if(date('H', $i) == 0){
                imageline($gImage, $x, $y - 3, $x, $y + 3, $gBlack);
                imagestring($gImage, $fontsize, $x + 30, $y + 2, date('D', $i), $gBlack);
            }
            break;
        case "month":
            if(date('w', $i) == 1){
                imageline($gImage, $x, $y - 3, $x, $y + 3, $gBlack);
                imagestring($gImage, $fontsize, $x + 40, $y + 2, "Week " . date('W', $i), $gBlack);
            }
            break;
        case "year":
            imageline($gImage, $x, $y - 3, $x, $y + 3, $gBlack);
            imagestring($gImage, $fontsize, $x + 16, $y + 2, date('M', $i + 86400 * 15), $gBlack);
            break;
        }
    }
    if($mag){
        $x = diagramPhysX(diagramScaleTime($mag));
        imageline($gImage, $x, $gHeight - $gBorder, $x, $gBorder, $gBlack);
    }
    imageline($gImage, $gWidth - $gBorder, $gBorder, $gWidth - $gBorder, $gHeight - $gBorder, $gBlack);
    imageline($gImage, $gBorder, $gHeight - $gBorder, $gBorder, $gBorder, $gBlack);
}


    diagramSetup(600, 250, 20);
    diagramAutoscaleSetup(0, 10, 30);   /* default range: 10 to 40 degrees */
    diagramAutoscaleSetup(1, 40, 60);   /* default range: 30 to 60 % rH */

    if(isset($_REQUEST['endTime'])){
        $endTime = $_REQUEST['endTime'];
    }else{
        $endTime = time();
    }
    if(isset($_REQUEST['interval'])){
        $interval = $_REQUEST['interval'];
    }else{
        $interval = 'day';
    }
    diagramDrawXAxis($endTime, $interval, 2);
    $files = array("day" => $csvfile, "week" => $avg30m, "month" => $avg2h, "year" => $avg1d);
    $file = $files[$interval];
    $fp = openAtTime($file, $gStartTime);
    if(!$fp){
        exit();
    }
    $data = array();
    while(($a = fgetcsv($fp, 1024, ";")) !== FALSE){
        if($file == $csvfile){
            $t = parseTimestamp($a[0]);
            if($t > $endTime)
                break;
            $sensor = (int)$a[1];
            if(!$_REQUEST["st$sensor"]){
                diagramAutoscaleForValue(0, $a[3]); /* temperature */
                $data[] = array($t, 2 * $sensor, $a[3]);
            }
            if($a[4] != 0 && !$_REQUEST["srh$sensor"]){ /* don't draw 0 humidity */
                diagramAutoscaleForValue(1, $a[4]); /* humidity */
                $data[] = array($t, 2 * $sensor + 1, $a[4]);
            }
        }else{
            if($a[0] > $endTime)
                break;
            for($i=0;$i<32;$i++){
                $val = $a[$i + 1];
                if($val != "*"){
                    $isHumidity = $i & 1;
                    $sensor = (int)($i / 2);
                    if($isHumidity){
                        $draw = $val != 0 && !$_REQUEST["srh$sensor"];  /* don't draw 0 humidity */
                    }else{
                        $draw = !$_REQUEST["st$sensor"];
                    }
                    if($draw){
                        diagramAutoscaleForValue($isHumidity, $val);
                        $data[] = array($a[0], $i, $val);
                    }
                }
            }
        }
    }
    fclose($fp);

    diagramDrawYAxis(0, "T/C", 2);
    diagramDrawYAxis(1, "rH/%", 2);

    for($i=0;$i<count($data);$i++){
        $a = $data[$i];
        diagramNextPoint(diagramScaleTime($a[0]), diagramAutoscaleValue($a[1] & 1, $a[2]), $a[1]);
    }

    header("Content-type: image/png");
//  header("Content-Length: " . filesize($path));
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");               // date from the past
    header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");  // always changed
    header("Cache-Control: no-store, no-cache, must-revalidate");   // HTTP/1.1
    header("Cache-Control: post-check=0, pre-check=0", false);      // second header, don't replace
    header("Pragma: no-cache");                                     // HTTP/1.0

    imagepng($gImage);
    imagedestroy($gImage);
?>
