/* Name: transmitter.c
 * Project: Remote Sensor; transmitter
 * Author: Christian Starkjohann
 * Creation Date: 2005-03-20
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: transmitter.c 312 2007-03-29 17:15:25Z cs $
 */

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "hardware.h"

#define UPDATE_FREQUENCY    37  /* in 8s units */

extern void uartTxBlock(char *block, unsigned char len);
extern unsigned crc16(unsigned char *data, unsigned char len);

volatile char   hadTimerInterrupt;

/* TX Frame:
 *  0: 2 bytes START OF FRAME (header)
 *  2: 1 byte unit address
 *  3: 1 byte numValues + sample ID
 *  4: 2 bytes value0 [battery]
 *  6: 2 bytes value1 [temperature]
 *  8: 2 bytes value2 [humidity]
 * 10: 2 bytes CRC
 */
static unsigned char    buffer[16];
static unsigned char    random;
static unsigned char    time8;
static unsigned char    isTestMode;

/* ------------------------------------------------------------------------- */

static inline void  adcOn(void)
{
    ADCSR = 1 << ADEN;  /* enable ADC, no conversion start, no interrupt, prescaler=0 */
}

static inline void  adcOff(void)
{
    ADCSR = 0;
}

void    adcConvert(unsigned char *result)
{
#ifndef NO_ADC_SLEEP
    ADCSR |= 1 << ADIE; /* enable ADC interrupt in order to wake up when result complete */
    MCUCR = (MCUCR & ~(1 << SM1)) | (1 << SM0) | (1 << SE); /* enable sleep, set sleep mode to ADC noise reduction */
    asm volatile("sleep" ::);   /* enter ADC noise reduction and start conversion */
#else
    ADCSR = (1 << ADEN) | (1 << ADSC);
    while(ADCSR & (1 << ADSC)); /* wait until conversion ready */
#endif
    /* Since we use no external interrupts, ADC ready is the only cause for a wakeup */
    result[0] = ADCL;
    result[1] = ADCH;
}

/* ------------------------------------------------------------------------- */

static void powerOff(void)
{
    TCCR1A = 0; /* no PWM */
    PLLCSR = 0; /* turn off PLL */
    ACSR = 1 << ACD;    /* disable analog comparator */
}

static void powerOn(void)
{
unsigned char   i = 0;

    ACSR = 1 << ACBG;   /* set reference to bandgap, comparator on */
    PLLCSR = (1 << PCKE) | (1 << PLLE); /* use 64MHz for PWM [6] */
    TCCR1A = (3 << COM1B0) | (1 << PWM1B);  /* PWM B [0x31] */
    while(ACSR & (1 << ACO)){   /* wait until nominal voltage reached */
        if(--i == 0)            /* timeout */
            break;
    }
    TCCR1A = 0; /* no PWM */
    PLLCSR = 0; /* turn off PLL */
}

#if 0
static void pollPower(void)
{
    if(ACSR & (1 << ACO)){
        PLLCSR = (1 << PCKE) | (1 << PLLE); /* use 64MHz for PWM [6] */
        TCCR1A = (3 << COM1B0) | (1 << PWM1B);  /* PWM B [0x31] */
    }else{
        TCCR1A = 0; /* no PWM */
        PLLCSR = 0; /* turn off PLL */
    }

}
#endif

/* ------------------------------------------------------------------------- */

#define DEVID_MASK_A    0x08
#define DEVID_MASK_B    0x46

static char getDeviceId(void)
{
/* Device-ID is encoded on pins PB6:PA3:PB2:PB1 */
unsigned char   a, b;

    /* activate pull-ups */
    PORTA |= DEVID_MASK_A;
    PORTB |= DEVID_MASK_B;
    a = PINA;
    b = PINB;
    /* remove pull-ups where pin is low; this saves power */
    PORTA &= a | ~DEVID_MASK_A;
    PORTB &= b | ~DEVID_MASK_B;
    b = ~b & DEVID_MASK_B;
    a = ~a & DEVID_MASK_A;
    return ((b | a) >> 1) | (b >> 3);
}

static void sampleValues(void)
{
unsigned char   *p = buffer + 4;
unsigned int    *val;

    adcOn();
    ADMUX = 0x1e;   /* ref = AVCC, right adjust, mux = 1.18V band gap */
    adcConvert(p);  /* sample bandgap ref -> battery */
    p += 2;

    PORTA |= 1 << TEMP_BIT_REF;
    ADMUX = 4;  /* ref = AVCC, right adjust, mux = ADC4 */
    adcConvert(p);  /* sample thermometer (differential, 20x, ADC4-ADC3) */
    PORTA &= ~(1 << TEMP_BIT_REF);
    random += *p;
    p += 2;

    ADMUX = 1;      /* ref = AVCC, right adjust, mux = ADC1 (humidity) */
    PORTA |= 1 << HUMID_BIT_REF;
    adcConvert(p);  /* sample humidity [+] */
    PORTA &= ~(1 << HUMID_BIT_REF);
    val = (void *)p;
    random += *p;
    p += 2; /* there is always room for one more sample (reserved for CRC) */
    PORTA |= 1 << HUMID_BIT_GND;
    adcConvert(p);  /* sample humidity [-] */
    PORTA &= ~(1 << HUMID_BIT_GND);

    adcOff();   /* we have all data now */
    random += *p;
    *val = (1024 - val[1] + val[0]) / 2;
}

static unsigned char    buildTxMsg(void)
{
static char     msgCount;
unsigned int    crc;

    sampleValues();
    buffer[0] = 0xdd;
    buffer[1] = 0x16;   /* Start Of Frame = 0xdd16 */
    buffer[2] = getDeviceId();
    buffer[3] = 0x30 | (msgCount & 0x0f);
    msgCount++;
    crc = crc16(buffer, 10);
    buffer[10] = (char)crc;
    buffer[11] = (char)(crc >> 8);
    return 12;
}

/* ------------------------------------------------------------------------- */

int main(void)
{
unsigned char   i, len;

    DDRA = 0;
    PORTA = 0xff;   /* all pull-ups */
    DDRB = 1 | (1 << 3);/* PB0 and PWM B as an output */
    PORTB = ~DDRB;      /* turn on pullups on inputs only, outputs default to 0 */
    if(!(PINA & (1<<TEMP_BIT_GND))){    /* test mode if TEMP_BIT_GND wired to low */
        isTestMode = 1;
    }
    DDRA = (1<<TEMP_BIT_REF) | (1<<TEMP_BIT_GND) | (1<<HUMID_BIT_REF) | (1<<HUMID_BIT_GND);
    PORTA = 1 << 3;     /* pullup only on unused input PA3 (analog reference) */
    ACSR |= 1 << ACD;   /* disable analog comparator */
    TCCR0 = 5;          /* prescaler = 1/1024 -> overflow interrupt interval = 8s */
    TIMSK = 1 << TOIE0; /* enable timer0 overflow interrupt */
    TCCR1A = 0;
    TCCR1B = (1 << CTC1) | 4;   /* reset on compare match, prescale = 1/8 */
    OCR1C = PWM_MAXVAL;
    OCR1B = PWM_VALUE;
    sei();
    for(;;){
        len = buildTxMsg();
        random += (random >> 4) + 1;
        powerOn();
        for(i=0;i<3;i++)
            uartTxBlock(buffer, len);
        powerOff();
        if(!isTestMode){
            MCUCR |= 1 << SE;
            MCUCR = (MCUCR | (1 << SE)) & ~(3 << SM0);  /* sleep = idle */
            while(time8 < UPDATE_FREQUENCY + (random & 0x7)){
                while(!hadTimerInterrupt)
                    asm volatile("sleep" ::);
                hadTimerInterrupt = 0;
                time8++;
            }
            time8 = 0;
        }
    }
}

/* ------------------------------------------------------------------------- */
